@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 """
@@ -12,11 +12,16 @@ Linux kernel networking subsystem.
 import operator
 from typing import Iterator, Union
 
-from drgn import NULL, IntegerLike, Object, Program
-from drgn.helpers.linux.list import hlist_for_each_entry
+from drgn import NULL, IntegerLike, Object, Program, cast, container_of
+from drgn.helpers.linux.fs import fget
+from drgn.helpers.linux.list import hlist_for_each_entry, list_for_each_entry
 from drgn.helpers.linux.list_nulls import hlist_nulls_for_each_entry
 
 __all__ = (
+    "for_each_net",
+    "get_net_ns_by_inode",
+    "get_net_ns_by_fd",
+    "netdev_for_each_tx_queue",
     "netdev_get_by_index",
     "netdev_get_by_name",
     "sk_fullsock",
@@ -24,6 +29,65 @@ __all__ = (
 )
 
 
+def for_each_net(prog: Program) -> Iterator[Object]:
+    """
+    Iterate over all network namespaces in the system.
+
+    :return: Iterator of ``struct net *`` objects.
+    """
+    for net in list_for_each_entry(
+        "struct net", prog["net_namespace_list"].address_of_(), "list"
+    ):
+        yield net
+
+
+_CLONE_NEWNET = 0x40000000
+
+
+def get_net_ns_by_inode(inode: Object) -> Object:
+    """
+    Get a network namespace from a network namespace NSFS inode, e.g.
+    ``/proc/$PID/ns/net`` or ``/var/run/netns/$NAME``.
+
+    :param inode: ``struct inode *``
+    :return: ``struct net *``
+    :raises ValueError: if *inode* is not a network namespace inode
+    """
+    if inode.i_fop != inode.prog_["ns_file_operations"].address_of_():
+        raise ValueError("not a namespace inode")
+
+    ns = cast("struct ns_common *", inode.i_private)
+    if ns.ops.type != _CLONE_NEWNET:
+        raise ValueError("not a network namespace inode")
+
+    return container_of(ns, "struct net", "ns")
+
+
+def get_net_ns_by_fd(task: Object, fd: IntegerLike) -> Object:
+    """
+    Get a network namespace from a task and a file descriptor referring to a
+    network namespace NSFS inode, e.g. ``/proc/$PID/ns/net`` or
+    ``/var/run/netns/$NAME``.
+
+    :param task: ``struct task_struct *``
+    :param fd: File descriptor.
+    :return: ``struct net *``
+    :raises ValueError: If *fd* does not refer to a network namespace inode
+    """
+    return get_net_ns_by_inode(fget(task, fd).f_inode)
+
+
+def netdev_for_each_tx_queue(dev: Object) -> Iterator[Object]:
+    """
+    Iterate over all TX queues for a network device.
+
+    :param dev: ``struct net_device *``
+    :return: Iterator of ``struct netdev_queue *`` objects.
+    """
+    for i in range(dev.num_tx_queues):
+        yield dev._tx + i
+
+
 _NETDEV_HASHBITS = 8
 _NETDEV_HASHENTRIES = 1 << _NETDEV_HASHBITS
 
