@@ -1,4 +1,4 @@
-// Copyright (c) Facebook, Inc. and its affiliates.
+// Copyright (c) Meta Platforms, Inc. and affiliates.
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
@@ -28,7 +28,6 @@
 #include "type.h"
 #include "vector.h"
 
-struct drgn_debug_info;
 struct drgn_symbol;
 
 /**
@@ -66,8 +65,8 @@ struct vmcoreinfo {
 };
 
 DEFINE_VECTOR_TYPE(drgn_typep_vector, struct drgn_type *)
-DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct string)
-DEFINE_HASH_MAP_TYPE(drgn_prstatus_map, uint32_t, struct string)
+DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct nstring)
+DEFINE_HASH_MAP_TYPE(drgn_prstatus_map, uint32_t, struct nstring)
 
 struct drgn_program {
 	/** @privatesection */
@@ -122,7 +121,7 @@ struct drgn_program {
 	 * Debugging information.
 	 */
 	struct drgn_object_index oindex;
-	struct drgn_debug_info *_dbinfo;
+	struct drgn_debug_info *dbinfo;
 
 	/*
 	 * Program information.
@@ -257,9 +256,6 @@ drgn_program_address_mask(const struct drgn_program *prog, uint64_t *ret)
 	return NULL;
 }
 
-struct drgn_error *drgn_program_get_dbinfo(struct drgn_program *prog,
-					   struct drgn_debug_info **ret);
-
 /**
  * Find the @c NT_PRSTATUS note for the given CPU.
  *
@@ -271,7 +267,7 @@ struct drgn_error *drgn_program_get_dbinfo(struct drgn_program *prog,
  */
 struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
 						     uint32_t cpu,
-						     struct string *ret,
+						     struct nstring *ret,
 						     uint32_t *tid_ret);
 
 /**
@@ -284,7 +280,7 @@ struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
  */
 struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
 						     uint32_t tid,
-						     struct string *ret);
+						     struct nstring *ret);
 
 /**
  * Cache the @c NT_PRSTATUS note provided by @p data in @p prog.
