@@ -1,4 +1,4 @@
-// Copyright (c) Facebook, Inc. and its affiliates.
+// Copyright (c) Meta Platforms, Inc. and affiliates.
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #ifndef DRGNPY_H
@@ -161,6 +161,8 @@ extern PyObject *PlatformFlags_class;
 extern PyObject *PrimitiveType_class;
 extern PyObject *ProgramFlags_class;
 extern PyObject *Qualifiers_class;
+extern PyObject *SymbolBinding_class;
+extern PyObject *SymbolKind_class;
 extern PyObject *TypeKind_class;
 extern PyTypeObject DrgnObject_type;
 extern PyTypeObject DrgnType_type;
@@ -291,8 +293,6 @@ DrgnObject *drgnpy_linux_helper_pid_task(PyObject *self, PyObject *args,
 					 PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args,
 					  PyObject *kwds);
-PyObject *drgnpy_linux_helper_task_state_to_char(PyObject *self, PyObject *args,
-						 PyObject *kwds);
 PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *args,
 					   PyObject *kwds);
 PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *args,
